@@ -1,5 +1,26 @@
 /* graph.h */
 /*
+    Copyright Vladimir Kolmogorov (vnk@ist.ac.at), Yuri Boykov (yuri@csd.uwo.ca) 
+
+    This file is part of MAXFLOW.
+
+    MAXFLOW is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    MAXFLOW is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with MAXFLOW.  If not, see <http://www.gnu.org/licenses/>.
+
+========================
+
+	version 3.04
+
 	This software library implements the maxflow algorithm
 	described in
 
@@ -29,6 +50,8 @@
 	If you use this option, you should cite
 	the aforementioned paper in any resulting publication.
 */
+	
+
 
 /*
 	For description, license, example usage see README.TXT.
@@ -56,17 +79,16 @@ typedef enum
 // flowtype: type of total flow
 //
 // Current instantiations are in instances.inc
-template <typename T, typename tcaptype, typename flowtype> class Graph
+template <typename captype, typename tcaptype, typename flowtype> class Graph
 {
 public:
 	typedef int node_id;
-        typedef T captype;
-        
+
 	/////////////////////////////////////////////////////////////////////////
 	//                     BASIC INTERFACE FUNCTIONS                       //
-    //              (should be enough for most applications)               //
+	//              (should be enough for most applications)               //
 	/////////////////////////////////////////////////////////////////////////
-	
+
 	// Constructor. 
 	// The first argument gives an estimate of the maximum number of nodes that can be added
 	// to the graph, and the second argument is an estimate of the maximum number of edges.
@@ -256,6 +278,7 @@ public:
 	void add_grid_tedges(PyArrayObject* nodeids,
             PyObject* sourcecaps, PyObject* sinkcaps);
 	PyArrayObject* get_grid_segments(PyArrayObject* nodeids);
+    void mark_grid_nodes(PyArrayObject* nodeids);
 	
 /////////////////////////////////////////////////////////////////////////
 /////////////////////////////////////////////////////////////////////////
@@ -271,7 +294,7 @@ private:
 		arc			*parent;	// node's parent
 		node		*next;		// pointer to the next active node
 								//   (or to itself if it is the last node in the list)
-		int			TS;			// timestamp showing when DIST was computed
+		long long			TS;			// timestamp showing when DIST was computed
 		int			DIST;		// distance to the terminal
 		int			is_sink : 1;	// flag showing whether the node is in the source or in the sink tree (if parent!=NULL)
 		int			is_marked : 1;	// set by mark_node()
@@ -319,7 +342,7 @@ private:
 
 	node				*queue_first[2], *queue_last[2];	// list of active nodes
 	nodeptr				*orphan_first, *orphan_last;		// list of pointers to orphans
-	int					TIME;								// monotonically increasing global counter
+	long long					TIME;								// monotonically increasing global counter
 
 	/////////////////////////////////////////////////////////////////////////
 
@@ -356,25 +379,12 @@ template <typename captype, typename tcaptype, typename flowtype>
 
 	if (node_last + num > node_max) reallocate_nodes(num);
 
-	if (num == 1)
-	{
-		node_last -> first = NULL;
-		node_last -> tr_cap = 0;
-		node_last -> is_marked = 0;
-		node_last -> is_in_changed_list = 0;
+	memset(node_last, 0, num*sizeof(node));
 
-		node_last ++;
-		return node_num ++;
-	}
-	else
-	{
-		memset(node_last, 0, num*sizeof(node));
-
-		node_id i = node_num;
-		node_num += num;
-		node_last += num;
-		return i;
-	}
+	node_id i = node_num;
+	node_num += num;
+	node_last += num;
+	return i;
 }
 
 template <typename captype, typename tcaptype, typename flowtype> 
@@ -436,20 +446,20 @@ template <typename captype, typename tcaptype, typename flowtype>
 }
 
 template <typename captype, typename tcaptype, typename flowtype> 
-	inline size_t Graph<captype,tcaptype,flowtype>::get_first_arc()
+	inline uintptr_t Graph<captype,tcaptype,flowtype>::get_first_arc()
 {
-	return reinterpret_cast<size_t>(arcs);
+	return reinterpret_cast<uintptr_t>(arcs);
 }
 
 template <typename captype, typename tcaptype, typename flowtype> 
-	inline size_t Graph<captype,tcaptype,flowtype>::get_next_arc(size_t _a) 
+	inline uintptr_t Graph<captype,tcaptype,flowtype>::get_next_arc(uintptr_t _a) 
 {
 	arc* a = reinterpret_cast<arc*>(_a);
-	return reinterpret_cast<size_t>(a + 1);
+	return reinterpret_cast<uintptr_t>(a + 1);
 }
 
 template <typename captype, typename tcaptype, typename flowtype> 
-	inline void Graph<captype,tcaptype,flowtype>::get_arc_ends(size_t _a, node_id& i, node_id& j)
+	inline void Graph<captype,tcaptype,flowtype>::get_arc_ends(arc_id _a, node_id& i, node_id& j)
 {
 	arc* a = reinterpret_cast<arc*>(_a);
 	assert(a >= arcs && a < arc_last);
@@ -517,6 +527,12 @@ template <typename captype, typename tcaptype, typename flowtype>
 	i->is_marked = 1;
 }
 
+/*
+	special constants for node->parent. Duplicated in maxflow.cpp, both should match!
+*/
+#define TERMINAL ( (arc *) 1 )		/* to terminal */
+#define ORPHAN   ( (arc *) 2 )		/* orphan */
+
 template <typename captype, typename tcaptype, typename flowtype> 
 	Graph<captype, tcaptype, flowtype>::Graph(int node_num_max, int edge_num_max, void (*err_function)(const char *))
 	: node_num(0),
@@ -584,7 +600,12 @@ template <typename captype, typename tcaptype, typename flowtype>
 
 	if (nodes != nodes_old)
 	{
+		node* i;
 		arc* a;
+		for (i=nodes; i<node_last; i++)
+		{
+			if (i->next) i->next = (node*) ((char*)i->next + (((char*) nodes) - ((char*) nodes_old)));
+		}
 		for (a=arcs; a<arc_last; a++)
 		{
 			a->head = (node*) ((char*)a->head + (((char*) nodes) - ((char*) nodes_old)));
@@ -613,6 +634,7 @@ template <typename captype, typename tcaptype, typename flowtype>
 		for (i=nodes; i<node_last; i++)
 		{
 			if (i->first) i->first = (arc*) ((char*)i->first + (((char*) arcs) - ((char*) arcs_old)));
+			if (i->parent && i->parent != ORPHAN && i->parent != TERMINAL) i->parent = (arc*) ((char*)i->parent + (((char*) arcs) - ((char*) arcs_old)));
 		}
 		for (a=arcs; a<arc_last; a++)
 		{
